@@ -28,4 +28,20 @@ fn main() {
     if version.semver >= Version::new(1, 50, 0) {
         println!("cargo:rustc-cfg=feature=\"since_1_50_0\""); // extended_compare_and_swap deprecated
     }
+
+    if version.semver >= Version::new(1, 53, 0) {
+        println!("cargo:rustc-cfg=feature=\"since_1_53_0\""); // fetch_update for AtomicBool and AtomicPtr
+    }
+
+    if version.semver >= Version::new(1, 60, 0) {
+        println!("cargo:rustc-cfg=feature=\"since_1_60_0\""); // target_has_atomic
+    }
+
+    if version.semver >= Version::new(1, 70, 0) {
+        println!("cargo:rustc-cfg=feature=\"since_1_70_0\""); // atomic_as_ptr
+    }
+
+    if version.semver >= Version::new(1, 75, 0) {
+        println!("cargo:rustc-cfg=feature=\"since_1_75_0\""); // atomic_from_ptr
+    }
 }
