@@ -337,7 +337,12 @@ impl ConnectorService {
     #[cfg(feature = "socks")]
     async fn connect_socks(&self, dst: Uri, proxy: ProxyScheme) -> Result<Conn, BoxError> {
         let dns = match proxy {
-            ProxyScheme::Socks4 { .. } => socks::DnsResolve::Local,
+            ProxyScheme::Socks4 {
+                remote_dns: false, ..
+            } => socks::DnsResolve::Local,
+            ProxyScheme::Socks4 {
+                remote_dns: true, ..
+            } => socks::DnsResolve::Proxy,
             ProxyScheme::Socks5 {
                 remote_dns: false, ..
             } => socks::DnsResolve::Local,
@@ -1157,7 +1162,6 @@ mod rustls_tls_conn {
 #[cfg(feature = "socks")]
 mod socks {
     use std::io;
-    use std::net::ToSocketAddrs;
 
     use http::Uri;
     use tokio::net::TcpStream;
@@ -1188,14 +1192,14 @@ mod socks {
         };
 
         if let DnsResolve::Local = dns {
-            let maybe_new_target = (host.as_str(), port).to_socket_addrs()?.next();
+            let maybe_new_target = tokio::net::lookup_host((host.as_str(), port)).await?.next();
             if let Some(new_target) = maybe_new_target {
                 host = new_target.ip().to_string();
             }
         }
 
         match proxy {
-            ProxyScheme::Socks4 { addr } => {
+            ProxyScheme::Socks4 { addr, .. } => {
                 let stream = Socks4Stream::connect(addr, (host.as_str(), port))
                     .await
                     .map_err(|e| format!("socks connect error: {e}"))?;
