@@ -4,6 +4,7 @@ use std::collections::{HashMap, HashSet};
 use std::ffi::{OsStr, OsString};
 use std::fmt;
 use std::path::Path;
+use std::process::ExitStatus;
 use std::str::FromStr;
 
 use crate::common::impl_get_set::impl_get_set;
@@ -91,6 +92,10 @@ impl System {
     /// Refreshes according to the given [`RefreshKind`]. It calls the corresponding
     /// "refresh_" methods.
     ///
+    /// It will remove dead processes if [`RefreshKind::processes`] returns `Some`.
+    /// If you want to keep dead processes, use [`System::refresh_processes_specifics`]
+    /// directly.
+    ///
     /// ```
     /// use sysinfo::{ProcessRefreshKind, RefreshKind, System};
     ///
@@ -109,7 +114,7 @@ impl System {
             self.refresh_cpu_specifics(kind);
         }
         if let Some(kind) = refreshes.processes() {
-            self.refresh_processes_specifics(ProcessesToUpdate::All, false, kind);
+            self.refresh_processes_specifics(ProcessesToUpdate::All, true, kind);
         }
     }
 
@@ -120,6 +125,9 @@ impl System {
     /// Don't forget to take a look at [`ProcessRefreshKind::everything`] method to see what it
     /// will update for processes more in details.
     ///
+    /// It will remove dead processes. If you want to keep dead processes, use
+    /// [`System::refresh_processes_specifics`] directly.
+    ///
     /// ```no_run
     /// use sysinfo::System;
     ///
@@ -255,7 +263,7 @@ impl System {
         self.inner.refresh_cpu_specifics(refresh_kind)
     }
 
-    /// Gets all processes and updates their information.
+    /// Gets all processes and updates their information, along with all the tasks each process has.
     ///
     /// It does the same as:
     ///
@@ -269,7 +277,7 @@ impl System {
     ///         .with_memory()
     ///         .with_cpu()
     ///         .with_disk_usage()
-    ///         .with_exe(UpdateKind::OnlyIfNotSet),
+    ///         .with_exe(UpdateKind::OnlyIfNotSet)
     /// );
     /// ```
     ///
@@ -280,6 +288,11 @@ impl System {
     /// ⚠️ On Linux, `sysinfo` keeps the `stat` files open by default. You can change this behaviour
     /// by using [`set_open_files_limit`][crate::set_open_files_limit].
     ///
+    /// ⚠️ On Linux, if you dont need the tasks of each process, you can use
+    /// `refresh_processes_specifics` with `ProcessRefreshKind::everything().without_tasks()`.
+    /// Refreshesing all processes and their tasks can be quite expensive. For more information
+    /// see [`ProcessRefreshKind`].
+    ///
     /// Example:
     ///
     /// ```no_run
@@ -300,7 +313,8 @@ impl System {
                 .with_memory()
                 .with_cpu()
                 .with_disk_usage()
-                .with_exe(UpdateKind::OnlyIfNotSet),
+                .with_exe(UpdateKind::OnlyIfNotSet)
+                .with_tasks(),
         )
     }
 
@@ -343,7 +357,9 @@ impl System {
         }
         fn update(pid: &Pid, processes: &mut HashMap<Pid, Process>) {
             if let Some(proc) = processes.get_mut(pid) {
-                proc.inner.switch_updated();
+                if !proc.inner.switch_updated() {
+                    proc.inner.set_nonexistent();
+                }
             }
         }
 
@@ -504,22 +520,6 @@ impl System {
         self.inner.cpus()
     }
 
-    /// Returns the number of physical cores on the CPU or `None` if it couldn't get it.
-    ///
-    /// In case there are multiple CPUs, it will combine the physical core count of all the CPUs.
-    ///
-    /// **Important**: this information is computed every time this function is called.
-    ///
-    /// ```no_run
-    /// use sysinfo::System;
-    ///
-    /// let s = System::new();
-    /// println!("{:?}", s.physical_core_count());
-    /// ```
-    pub fn physical_core_count(&self) -> Option<usize> {
-        self.inner.physical_core_count()
-    }
-
     /// Returns the RAM size in bytes.
     ///
     /// ```no_run
@@ -792,6 +792,68 @@ impl System {
         SystemInner::distribution_id()
     }
 
+    /// Returns the distribution ids of operating systems that are closely
+    /// related to the local operating system in regards to packaging and
+    /// programming interfaces, for example listing one or more OS identifiers
+    /// the local OS is a derivative from.
+    ///
+    /// See also
+    /// - <https://www.freedesktop.org/software/systemd/man/latest/os-release.html#ID_LIKE=>
+    ///
+    /// | example platform | value of `System::distribution_id_like()` |
+    /// |---|---|
+    /// | android phone | [] |
+    /// | archlinux laptop | [] |
+    /// | centos server | ["rhel", "fedora"] |
+    /// | ubuntu laptop | ["debian"] |
+    /// | windows laptop | [] |
+    ///
+    /// **Important**: this information is computed every time this function is called.
+    ///
+    /// ```no_run
+    /// use sysinfo::System;
+    ///
+    /// println!("Distribution ID_LIKE: {:?}", System::distribution_id_like());
+    /// ```
+    pub fn distribution_id_like() -> Vec<String> {
+        SystemInner::distribution_id_like()
+    }
+
+    /// Provides kernel version following this string format:
+    ///
+    /// | Platform | Result |
+    /// |-|-|
+    /// | Windows | Windows OS Build 20348.2227 |
+    /// | Linux | Linux 6.12.13-200.fc41.x86_64 |
+    /// | Android | Android 612.13-200 |
+    /// | MacOS | Darwin 21.6.0 |
+    /// | FreeBSD | FreeBSD 199506 |
+    ///
+    /// If any of the information is not available, it will be replaced with "unknown".
+    ///
+    /// **Important**: this information is computed every time this function is called.
+    ///
+    /// ```no_run
+    /// use sysinfo::System;
+    ///
+    /// println!("Kernel long version: {}", System::kernel_long_version());
+    /// ```
+    ///
+    /// [distribution_id]: System::distribution_id
+    /// [kernel_version]: System::kernel_version
+    pub fn kernel_long_version() -> String {
+        let kernel_version = match System::kernel_version() {
+            None => "unknown".to_string(),
+            Some(s) => s,
+        };
+        let kernel_name = SystemInner::kernel_name().unwrap_or("Unknown");
+        if cfg!(windows) {
+            format!("{kernel_name} OS Build {kernel_version}")
+        } else {
+            format!("{kernel_name} {kernel_version}")
+        }
+    }
+
     /// Returns the system hostname based off DNS.
     ///
     /// **Important**: this information is computed every time this function is called.
@@ -817,6 +879,22 @@ impl System {
     pub fn cpu_arch() -> String {
         SystemInner::cpu_arch().unwrap_or_else(|| std::env::consts::ARCH.to_owned())
     }
+
+    /// Returns the number of physical cores on the CPU or `None` if it couldn't get it.
+    ///
+    /// In case there are multiple CPUs, it will combine the physical core count of all the CPUs.
+    ///
+    /// **Important**: this information is computed every time this function is called.
+    ///
+    /// ```no_run
+    /// use sysinfo::System;
+    ///
+    /// let s = System::new();
+    /// println!("{:?}", System::physical_core_count());
+    /// ```
+    pub fn physical_core_count() -> Option<usize> {
+        SystemInner::physical_core_count()
+    }
 }
 
 /// A struct representing system load average value.
@@ -852,7 +930,8 @@ pub struct LoadAvg {
 ///
 /// If you want the list of the supported signals on the current system, use
 /// [`SUPPORTED_SIGNALS`][crate::SUPPORTED_SIGNALS].
-#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Debug)]
+#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, PartialOrd, Ord)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
 pub enum Signal {
     /// Hangup detected on controlling terminal or death of controlling process.
     Hangup,
@@ -976,7 +1055,8 @@ pub struct CGroupLimits {
 }
 
 /// Enum describing the different status of a process.
-#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, PartialOrd, Ord)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
 pub enum ProcessStatus {
     /// ## Linux
     ///
@@ -1094,6 +1174,7 @@ pub enum ProcessStatus {
 
 /// Enum describing the different kind of threads.
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
 pub enum ThreadKind {
     /// Kernel thread.
     Kernel,
@@ -1181,7 +1262,8 @@ impl Process {
         self.inner.kill_with(signal)
     }
 
-    /// Wait for process termination.
+    /// Wait for process termination and returns its [`ExitStatus`] if it could be retrieved,
+    /// returns `None` otherwise.
     ///
     /// ```no_run
     /// use sysinfo::{Pid, System};
@@ -1190,11 +1272,11 @@ impl Process {
     ///
     /// if let Some(process) = s.process(Pid::from(1337)) {
     ///     println!("Waiting for pid 1337");
-    ///     process.wait();
-    ///     println!("Pid 1337 exited");
+    ///     let exit_status = process.wait();
+    ///     println!("Pid 1337 exited with: {exit_status:?}");
     /// }
     /// ```
-    pub fn wait(&self) {
+    pub fn wait(&self) -> Option<ExitStatus> {
         self.inner.wait()
     }
 
@@ -1468,6 +1550,22 @@ impl Process {
         self.inner.cpu_usage()
     }
 
+    /// Returns the total accumulated CPU usage (in CPU-milliseconds). Note
+    /// that it might be bigger than the total clock run time of a process if
+    /// run on a multi-core machine.
+    ///
+    /// ```no_run
+    /// use sysinfo::{Pid, System};
+    ///
+    /// let s = System::new_all();
+    /// if let Some(process) = s.process(Pid::from(1337)) {
+    ///     println!("{}", process.accumulated_cpu_time());
+    /// }
+    /// ```
+    pub fn accumulated_cpu_time(&self) -> u64 {
+        self.inner.accumulated_cpu_time()
+    }
+
     /// Returns number of bytes read and written to disk.
     ///
     /// ⚠️ On Windows, this method actually returns **ALL** I/O read and
@@ -1660,6 +1758,78 @@ impl Process {
             }
         }
     }
+
+    /// Returns `true` if the process doesn't exist anymore but was not yet removed from
+    /// the processes list because the `remove_dead_processes` argument was set to `false`
+    /// in methods like [`System::refresh_processes`].
+    ///
+    /// ```no_run
+    /// use sysinfo::{ProcessesToUpdate, System};
+    ///
+    /// let mut s = System::new_all();
+    /// // We set the `remove_dead_processes` to `false`.
+    /// s.refresh_processes(ProcessesToUpdate::All, false);
+    ///
+    /// for (_, process) in s.processes() {
+    ///     println!(
+    ///         "Process {:?} {}",
+    ///         process.pid(),
+    ///         if process.exists() { "exists" } else { "doesn't exist" },
+    ///     );
+    /// }
+    /// ```
+    pub fn exists(&self) -> bool {
+        self.inner.exists()
+    }
+
+    /// Returns the number of open files in the current process.
+    ///
+    /// Returns `None` if it failed retrieving the information or if the current system is not
+    /// supported.
+    ///
+    /// **Important**: this information is computed every time this function is called (except on
+    /// FreeBSD).
+    ///
+    /// ```no_run
+    /// use sysinfo::System;
+    ///
+    /// let s = System::new_all();
+    ///
+    /// for (_, process) in s.processes() {
+    ///     println!(
+    ///         "Process {:?} {:?}",
+    ///         process.pid(),
+    ///         process.open_files(),
+    ///     );
+    /// }
+    /// ```
+    pub fn open_files(&self) -> Option<u32> {
+        self.inner.open_files()
+    }
+
+    /// Returns the maximum number of open files for the current process.
+    ///
+    /// Returns `None` if it failed retrieving the information or if the current system is not
+    /// supported.
+    ///
+    /// **Important**: this information is computed every time this function is called.
+    ///
+    /// ```no_run
+    /// use sysinfo::System;
+    ///
+    /// let s = System::new_all();
+    ///
+    /// for (_, process) in s.processes() {
+    ///     println!(
+    ///         "Process {:?} {:?}",
+    ///         process.pid(),
+    ///         process.open_files_limit(),
+    ///     );
+    /// }
+    /// ```
+    pub fn open_files_limit(&self) -> Option<u32> {
+        self.inner.open_files_limit()
+    }
 }
 
 macro_rules! pid_decl {
@@ -1813,6 +1983,15 @@ pub enum ProcessesToUpdate<'a> {
 /// the information won't be retrieved if the information is accessible without needing
 /// extra computation.
 ///
+/// ⚠️ ** Linux Specific ** ⚠️
+/// When using `ProcessRefreshKind::everything()`, in linux we will fetch all relevant
+/// information from `/proc/<pid>/` as well as all the information from `/proc/<pid>/task/<tid>/`
+/// folders. This makes the refresh mechanism a lot slower depending on the number of tasks
+/// each process has.
+///  
+/// If you don't care about tasks information, use `ProcessRefreshKind::everything().without_tasks()`
+/// as much as possible.
+///
 /// ```
 /// use sysinfo::{ProcessesToUpdate, ProcessRefreshKind, System};
 ///
@@ -1832,7 +2011,7 @@ pub enum ProcessesToUpdate<'a> {
 /// ```
 ///
 /// [`Process`]: crate::Process
-#[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub struct ProcessRefreshKind {
     cpu: bool,
     disk_usage: bool,
@@ -1843,11 +2022,35 @@ pub struct ProcessRefreshKind {
     environ: UpdateKind,
     cmd: UpdateKind,
     exe: UpdateKind,
+    tasks: bool,
+}
+
+/// Creates a new `ProcessRefreshKind` with every refresh set to `false`, except for `tasks`.
+/// By default, we want to list all processes and tasks are considered processes on their own
+/// in linux so we still fetch them by default. However, the processes information are not
+/// refreshed.
+impl Default for ProcessRefreshKind {
+    fn default() -> Self {
+        Self {
+            cpu: false,
+            disk_usage: false,
+            memory: false,
+            user: UpdateKind::default(),
+            cwd: UpdateKind::default(),
+            root: UpdateKind::default(),
+            environ: UpdateKind::default(),
+            cmd: UpdateKind::default(),
+            exe: UpdateKind::default(),
+            tasks: true, // Process by default includes all tasks.
+        }
+    }
 }
 
 impl ProcessRefreshKind {
-    /// Creates a new `ProcessRefreshKind` with every refresh set to `false`.
-    ///
+    /// Creates a new `ProcessRefreshKind` with every refresh set to `false`, except for `tasks`.
+    /// By default, we want to list all processes and tasks are considered processes on their own
+    /// in linux so we still fetch them by default. However, the processes information are not
+    /// refreshed.
     /// ```
     /// use sysinfo::{ProcessRefreshKind, UpdateKind};
     ///
@@ -1882,10 +2085,18 @@ impl ProcessRefreshKind {
             environ: UpdateKind::OnlyIfNotSet,
             cmd: UpdateKind::OnlyIfNotSet,
             exe: UpdateKind::OnlyIfNotSet,
+            tasks: true,
         }
     }
 
-    impl_get_set!(ProcessRefreshKind, cpu, with_cpu, without_cpu);
+    impl_get_set!(
+        ProcessRefreshKind,
+        cpu,
+        with_cpu,
+        without_cpu,
+        "\
+It will retrieve both CPU usage and CPU accumulated time,"
+    );
     impl_get_set!(
         ProcessRefreshKind,
         disk_usage,
@@ -1924,6 +2135,7 @@ It will retrieve the following information:
     );
     impl_get_set!(ProcessRefreshKind, cmd, with_cmd, without_cmd, UpdateKind);
     impl_get_set!(ProcessRefreshKind, exe, with_exe, without_exe, UpdateKind);
+    impl_get_set!(ProcessRefreshKind, tasks, with_tasks, without_tasks);
 }
 
 /// Used to determine what you want to refresh specifically on the [`Cpu`] type.
@@ -2498,9 +2710,8 @@ mod test {
         if IS_SUPPORTED_SYSTEM {
             // The physical cores count is recomputed every time the function is called, so the
             // information must be relevant even with nothing initialized.
-            let physical_cores_count = s
-                .physical_core_count()
-                .expect("failed to get number of physical cores");
+            let physical_cores_count =
+                System::physical_core_count().expect("failed to get number of physical cores");
 
             s.refresh_cpu_usage();
             // The cpus shouldn't be empty anymore.
@@ -2508,15 +2719,14 @@ mod test {
 
             // In case we are running inside a VM, it's possible to not have a physical core, only
             // logical ones, which is why we don't test `physical_cores_count > 0`.
-            let physical_cores_count2 = s
-                .physical_core_count()
-                .expect("failed to get number of physical cores");
+            let physical_cores_count2 =
+                System::physical_core_count().expect("failed to get number of physical cores");
             assert!(physical_cores_count2 <= s.cpus().len());
             assert_eq!(physical_cores_count, physical_cores_count2);
         } else {
-            assert_eq!(s.physical_core_count(), None);
+            assert_eq!(System::physical_core_count(), None);
         }
-        assert!(s.physical_core_count().unwrap_or(0) <= s.cpus().len());
+        assert!(System::physical_core_count().unwrap_or(0) <= s.cpus().len());
     }
 
     // This test only exists to ensure that the `Display` and `Debug` traits are implemented on the
