@@ -5,13 +5,17 @@ use crate::sys::cpu::*;
 use crate::sys::process::*;
 use crate::sys::utils::{get_sys_value, get_sys_value_by_name};
 
-use crate::{Cpu, CpuRefreshKind, LoadAvg, MemoryRefreshKind, Pid, Process, ProcessRefreshKind};
+use crate::{
+    Cpu, CpuRefreshKind, LoadAvg, MemoryRefreshKind, Pid, Process, ProcessRefreshKind,
+    ProcessesToUpdate,
+};
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 use std::cell::UnsafeCell;
 use std::collections::HashMap;
 use std::ffi::CStr;
 use std::mem;
+use std::time::Duration;
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 use std::time::SystemTime;
 
@@ -20,6 +24,55 @@ use libc::{
     _SC_PAGESIZE,
 };
 
+#[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
+declare_signals! {
+    libc::c_int,
+    Signal::Hangup => libc::SIGHUP,
+    Signal::Interrupt => libc::SIGINT,
+    Signal::Quit => libc::SIGQUIT,
+    Signal::Illegal => libc::SIGILL,
+    Signal::Trap => libc::SIGTRAP,
+    Signal::Abort => libc::SIGABRT,
+    Signal::IOT => libc::SIGIOT,
+    Signal::Bus => libc::SIGBUS,
+    Signal::FloatingPointException => libc::SIGFPE,
+    Signal::Kill => libc::SIGKILL,
+    Signal::User1 => libc::SIGUSR1,
+    Signal::Segv => libc::SIGSEGV,
+    Signal::User2 => libc::SIGUSR2,
+    Signal::Pipe => libc::SIGPIPE,
+    Signal::Alarm => libc::SIGALRM,
+    Signal::Term => libc::SIGTERM,
+    Signal::Child => libc::SIGCHLD,
+    Signal::Continue => libc::SIGCONT,
+    Signal::Stop => libc::SIGSTOP,
+    Signal::TSTP => libc::SIGTSTP,
+    Signal::TTIN => libc::SIGTTIN,
+    Signal::TTOU => libc::SIGTTOU,
+    Signal::Urgent => libc::SIGURG,
+    Signal::XCPU => libc::SIGXCPU,
+    Signal::XFSZ => libc::SIGXFSZ,
+    Signal::VirtualAlarm => libc::SIGVTALRM,
+    Signal::Profiling => libc::SIGPROF,
+    Signal::Winch => libc::SIGWINCH,
+    Signal::IO => libc::SIGIO,
+    // SIGPOLL doesn't exist on apple targets but since it's an equivalent of SIGIO on unix,
+    // we simply use the SIGIO constant.
+    Signal::Poll => libc::SIGIO,
+    Signal::Sys => libc::SIGSYS,
+    _ => None,
+}
+#[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
+declare_signals! {
+    libc::c_int,
+    _ => None,
+}
+
+#[doc = include_str!("../../../md_doc/supported_signals.md")]
+pub const SUPPORTED_SIGNALS: &[crate::Signal] = supported_signals();
+#[doc = include_str!("../../../md_doc/minimum_cpu_update_interval.md")]
+pub const MINIMUM_CPU_UPDATE_INTERVAL: Duration = Duration::from_millis(200);
+
 pub(crate) struct SystemInner {
     process_list: HashMap<Pid, Process>,
     mem_total: u64,
@@ -39,9 +92,9 @@ pub(crate) struct SystemInner {
 pub(crate) struct Wrap<'a>(pub UnsafeCell<&'a mut HashMap<Pid, Process>>);
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-unsafe impl<'a> Send for Wrap<'a> {}
+unsafe impl Send for Wrap<'_> {}
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-unsafe impl<'a> Sync for Wrap<'a> {}
+unsafe impl Sync for Wrap<'_> {}
 
 fn boot_time() -> u64 {
     let mut boot_time = timeval {
@@ -79,6 +132,7 @@ fn get_now() -> u64 {
 impl SystemInner {
     pub(crate) fn new() -> Self {
         unsafe {
+            #[allow(deprecated)]
             let port = libc::mach_host_self();
 
             Self {
@@ -169,26 +223,33 @@ impl SystemInner {
         self.cpus.refresh(refresh_kind, self.port);
     }
 
+    pub(crate) fn refresh_cpu_list(&mut self, refresh_kind: CpuRefreshKind) {
+        self.cpus = CpusWrapper::new();
+        self.cpus.refresh(refresh_kind, self.port);
+    }
+
     #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
     pub(crate) fn refresh_processes_specifics(
         &mut self,
-        _filter: Option<&[Pid]>,
+        _processes_to_update: ProcessesToUpdate<'_>,
         _refresh_kind: ProcessRefreshKind,
-    ) {
+    ) -> usize {
+        0
     }
 
     #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
     pub(crate) fn refresh_processes_specifics(
         &mut self,
-        filter: Option<&[Pid]>,
+        processes_to_update: ProcessesToUpdate<'_>,
         refresh_kind: ProcessRefreshKind,
-    ) {
+    ) -> usize {
         use crate::utils::into_iter;
+        use std::sync::atomic::{AtomicUsize, Ordering};
 
         unsafe {
             let count = libc::proc_listallpids(::std::ptr::null_mut(), 0);
             if count < 1 {
-                return;
+                return 0;
             }
         }
         if let Some(pids) = get_proc_list() {
@@ -206,15 +267,25 @@ impl SystemInner {
             let (filter, filter_callback): (
                 &[Pid],
                 &(dyn Fn(Pid, &[Pid]) -> bool + Sync + Send),
-            ) = if let Some(filter) = filter {
-                (filter, &real_filter)
-            } else {
-                (&[], &empty_filter)
+            ) = match processes_to_update {
+                ProcessesToUpdate::All => (&[], &empty_filter),
+                ProcessesToUpdate::Some(pids_to_refresh) => {
+                    if pids_to_refresh.is_empty() {
+                        return 0;
+                    }
+                    (pids_to_refresh, &real_filter)
+                }
             };
 
+            let nb_updated = AtomicUsize::new(0);
             let now = get_now();
             let port = self.port;
             let time_interval = self.clock_info.as_mut().map(|c| c.get_time_interval(port));
+            let timebase_to_ms = self
+                .clock_info
+                .as_ref()
+                .map(|c| c.timebase_to_ms)
+                .unwrap_or_default();
             let entries: Vec<Process> = {
                 let wrap = &Wrap(UnsafeCell::new(&mut self.process_list));
 
@@ -226,51 +297,26 @@ impl SystemInner {
                         if !filter_callback(pid, filter) {
                             return None;
                         }
-                        match update_process(wrap, pid, time_interval, now, refresh_kind, false) {
-                            Ok(x) => x,
-                            _ => None,
-                        }
+                        nb_updated.fetch_add(1, Ordering::Relaxed);
+                        update_process(
+                            wrap,
+                            pid,
+                            time_interval,
+                            now,
+                            refresh_kind,
+                            false,
+                            timebase_to_ms,
+                        )
+                        .unwrap_or_default()
                     })
                     .collect()
             };
             entries.into_iter().for_each(|entry| {
                 self.process_list.insert(entry.pid(), entry);
             });
-            self.process_list
-                .retain(|_, proc_| std::mem::replace(&mut proc_.inner.updated, false));
-        }
-    }
-
-    #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
-    pub(crate) fn refresh_process_specifics(
-        &mut self,
-        _pid: Pid,
-        _refresh_kind: ProcessRefreshKind,
-    ) -> bool {
-        false
-    }
-
-    #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-    pub(crate) fn refresh_process_specifics(
-        &mut self,
-        pid: Pid,
-        refresh_kind: ProcessRefreshKind,
-    ) -> bool {
-        let mut time_interval = None;
-        let now = get_now();
-
-        if refresh_kind.cpu() {
-            let port = self.port;
-            time_interval = self.clock_info.as_mut().map(|c| c.get_time_interval(port));
-        }
-        let wrap = Wrap(UnsafeCell::new(&mut self.process_list));
-        match update_process(&wrap, pid, time_interval, now, refresh_kind, true) {
-            Ok(Some(p)) => {
-                self.process_list.insert(p.pid(), p);
-                true
-            }
-            Ok(_) => true,
-            Err(_) => false,
+            nb_updated.into_inner()
+        } else {
+            0
         }
     }
 
@@ -282,22 +328,22 @@ impl SystemInner {
         &self.process_list
     }
 
+    pub(crate) fn processes_mut(&mut self) -> &mut HashMap<Pid, Process> {
+        &mut self.process_list
+    }
+
     pub(crate) fn process(&self, pid: Pid) -> Option<&Process> {
         self.process_list.get(&pid)
     }
 
-    pub(crate) fn global_cpu_info(&self) -> &Cpu {
-        &self.cpus.global_cpu
+    pub(crate) fn global_cpu_usage(&self) -> f32 {
+        self.cpus.global_cpu.percent()
     }
 
     pub(crate) fn cpus(&self) -> &[Cpu] {
         &self.cpus.cpus
     }
 
-    pub(crate) fn physical_core_count(&self) -> Option<usize> {
-        physical_core_count()
-    }
-
     pub(crate) fn total_memory(&self) -> u64 {
         self.mem_total
     }
@@ -358,45 +404,53 @@ impl SystemInner {
 
     pub(crate) fn long_os_version() -> Option<String> {
         #[cfg(target_os = "macos")]
-        let friendly_name = match Self::os_version().unwrap_or_default() {
-            f_n if f_n.starts_with("14.0") => "Sonoma",
-            f_n if f_n.starts_with("10.16")
-                | f_n.starts_with("11.0")
-                | f_n.starts_with("11.1")
-                | f_n.starts_with("11.2") =>
-            {
-                "Big Sur"
+        {
+            let Some(os_version) = Self::os_version() else {
+                return Some("macOS".to_owned());
+            };
+            // https://en.wikipedia.org/wiki/MacOS_version_history
+            for (version_prefix, macos_spelling, friendly_name) in [
+                ("15", "macOS", "Sequoia"),
+                ("14", "macOS", "Sonoma"),
+                ("13", "macOS", "Ventura"),
+                ("12", "macOS", "Monterey"),
+                ("11", "macOS", "Big Sur"),
+                // Big Sur identifies itself as 10.16 in some situations.
+                // https://en.wikipedia.org/wiki/MacOS_Big_Sur#Development_history
+                ("10.16", "macOS", "Big Sur"),
+                ("10.15", "macOS", "Catalina"),
+                ("10.14", "macOS", "Mojave"),
+                ("10.13", "macOS", "High Sierra"),
+                ("10.12", "macOS", "Sierra"),
+                ("10.11", "OS X", "El Capitan"),
+                ("10.10", "OS X", "Yosemite"),
+                ("10.9", "OS X", "Mavericks"),
+                ("10.8", "OS X", "Mountain Lion"),
+                ("10.7", "Mac OS X", "Lion"),
+                ("10.6", "Mac OS X", "Snow Leopard"),
+                ("10.5", "Mac OS X", "Leopard"),
+                ("10.4", "Mac OS X", "Tiger"),
+                ("10.3", "Mac OS X", "Panther"),
+                ("10.2", "Mac OS X", "Jaguar"),
+                ("10.1", "Mac OS X", "Puma"),
+                ("10.0", "Mac OS X", "Cheetah"),
+            ] {
+                if os_version.starts_with(version_prefix) {
+                    return Some(format!("{macos_spelling} {os_version} {friendly_name}"));
+                }
             }
-            f_n if f_n.starts_with("10.15") => "Catalina",
-            f_n if f_n.starts_with("10.14") => "Mojave",
-            f_n if f_n.starts_with("10.13") => "High Sierra",
-            f_n if f_n.starts_with("10.12") => "Sierra",
-            f_n if f_n.starts_with("10.11") => "El Capitan",
-            f_n if f_n.starts_with("10.10") => "Yosemite",
-            f_n if f_n.starts_with("10.9") => "Mavericks",
-            f_n if f_n.starts_with("10.8") => "Mountain Lion",
-            f_n if f_n.starts_with("10.7") => "Lion",
-            f_n if f_n.starts_with("10.6") => "Snow Leopard",
-            f_n if f_n.starts_with("10.5") => "Leopard",
-            f_n if f_n.starts_with("10.4") => "Tiger",
-            f_n if f_n.starts_with("10.3") => "Panther",
-            f_n if f_n.starts_with("10.2") => "Jaguar",
-            f_n if f_n.starts_with("10.1") => "Puma",
-            f_n if f_n.starts_with("10.0") => "Cheetah",
-            _ => "",
-        };
-
-        #[cfg(target_os = "macos")]
-        let long_name = Some(format!(
-            "MacOS {} {}",
-            Self::os_version().unwrap_or_default(),
-            friendly_name
-        ));
+            Some(format!("macOS {os_version}"))
+        }
 
         #[cfg(target_os = "ios")]
-        let long_name = Some(format!("iOS {}", Self::os_version().unwrap_or_default()));
-
-        long_name
+        {
+            let mut long_name = "iOS".to_owned();
+            if let Some(os_version) = Self::os_version() {
+                long_name.push(' ');
+                long_name.push_str(&os_version);
+            }
+            Some(long_name)
+        }
     }
 
     pub(crate) fn host_name() -> Option<String> {
@@ -443,6 +497,14 @@ impl SystemInner {
         std::env::consts::OS.to_owned()
     }
 
+    pub(crate) fn distribution_id_like() -> Vec<String> {
+        Vec::new()
+    }
+
+    pub(crate) fn kernel_name() -> Option<&'static str> {
+        Some("Darwin")
+    }
+
     pub(crate) fn cpu_arch() -> Option<String> {
         let mut arch_str: [u8; 32] = [0; 32];
         let mut mib = [libc::CTL_HW as _, libc::HW_MACHINE as _];
@@ -464,6 +526,10 @@ impl SystemInner {
             }
         }
     }
+
+    pub(crate) fn physical_core_count() -> Option<usize> {
+        physical_core_count()
+    }
 }
 
 fn get_system_info(value: c_int, default: Option<&str>) -> Option<String> {
