@@ -1144,6 +1144,47 @@ pub(crate) mod test {
         assert!(matches!(err, Error::LargeResponseHeader(65, 5)));
     }
 
+    #[test]
+    fn propfind_with_body() {
+        init_test_log();
+
+        // https://github.com/algesten/ureq/issues/1034
+        let request = http::Request::builder()
+            .method("PROPFIND")
+            .uri("https://www.google.com/")
+            .body("Some really cool body")
+            .unwrap();
+
+        let _ = Agent::config_builder()
+            .allow_non_standard_methods(true)
+            .build()
+            .new_agent()
+            .run(request)
+            .unwrap();
+    }
+
+    #[test]
+    #[cfg(feature = "_test")]
+    fn non_standard_method() {
+        init_test_log();
+        let method = Method::from_bytes(b"FNORD").unwrap();
+
+        let req = Request::builder()
+            .method(method)
+            .uri("http://httpbin.org/fnord")
+            .body(())
+            .unwrap();
+
+        let agent = Agent::new_with_defaults();
+
+        let req = agent
+            .configure_request(req)
+            .allow_non_standard_methods(true)
+            .build();
+
+        agent.run(req).unwrap();
+    }
+
     // This doesn't need to run, just compile.
     fn _ensure_send_sync() {
         fn is_send(_t: impl Send) {}
@@ -1185,5 +1226,10 @@ pub(crate) mod test {
         let response = post("https://yaz").send(&data).unwrap();
         let owned_reader = response.into_parts().1.into_reader();
         is_sync(owned_reader);
+
+        let err = Error::HostNotFound;
+        is_send(err);
+        let err = Error::HostNotFound;
+        is_sync(err);
     }
 }
